/* program to extract the experimental conditions from a cad4 data file.
 * This is namely, records 21-27 and 31-32 at the start of each file.
 * Program takes an fopen() file stream pointing to a CAD4 data file.
 * 	Paul D. Boyle
 */
#define _POSIX_SOURCE

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* local headers */
#include "fildat.h"

#define  GOT_REC_21 (1 << 0)
#define  GOT_REC_22 (1 << 1)
#define  GOT_REC_23 (1 << 2)
#define  GOT_REC_24 (1 << 3)
#define  GOT_REC_25 (1 << 4)
#define  GOT_REC_26 (1 << 5)
#define  GOT_REC_27 (1 << 6)
#define  GOT_REC_31 (1 << 7)
#define  GOT_REC_32 (1 << 8)

#define GOT_ALL 0x01ff

void GetCAD4Experimental( FILE *cad4_file, struct cad4_experimental *exper )
{
    char                     read_in[CAD4_LINE_LEN], 
                             junk[CAD4_LINE_LEN],
                             *ptr, 
                             mystery1, mystery2;
    int record,
        ijunk,
        ijunk_27;
    unsigned short rec_flg = 0;

    float fjunk_27[4];

    while( NULL != fgets(read_in, CAD4_LINE_LEN, cad4_file) ) {
        if( NULL != (ptr = strchr(read_in, '\n')) ) 
            *ptr = '\0';
        if( NULL != (ptr = strchr(read_in, '\r')) ) 
            *ptr = '\0';
/* check to see that we are getting data file records rather than the
 * listing of the fcryst file reflection list at the beginning of the
 * data file.  Initially, check to see if we have all records.  If so,
 * break out of the loop.
 */     
        if( GOT_ALL == rec_flg ) break;
   
        strncpy( junk, read_in, 4 );
        junk[4] = '\0';
        if( NULL == strpbrk( junk, "0123456789" ) ) continue;

        sscanf( read_in, "%d %s", &record, junk );
        switch( record ) {
          case 21:
             sscanf( read_in, "%d %40s %f %f %f %f", &ijunk,
                     exper->text, &exper->instr_radius,
                     &exper->vertical_slit, &exper->apeture_min,
                     &exper->apeture_max );

             rec_flg |= GOT_REC_21;
             break;
          case 22:
             sscanf( read_in, "%d %f %f %f %f %f %f %d %f %f %d %d %d %d %d",
                     &ijunk, &exper->theta_min, &exper->theta_max, 
                     &exper->omega_term_a, &exper->omega_term_b,
                     &exper->apeture_set_a, &exper->apeture_set_b, 
                     &exper->scan_mode, &exper->sigma_prescan,
                     &exper->sigma_desired, &exper->npi_pre, &exper->max_time,
                     &exper->dumpfl, &exper->n_balf, &exper->n_fridl );

             rec_flg |= GOT_REC_22;
             break;
          case 23:
             sscanf( read_in, "%d %d %6s %d %f %f %f %f %f", &ijunk,
                     &exper->n_eqfl, exper->psi_word, &exper->nr_psi, 
                     &exper->psi_start, &exper->psi_delta, 
                     &exper->vect[0], &exper->vect[1], &exper->vect[2] );

             rec_flg |= GOT_REC_23;
             break;
          case 24:
             sscanf( read_in, "%d %6s %3s %d %d %d %d %d %d %d %d %d", 
                     &ijunk, exper->mode_word, exper->seq, &exper->h_start,
                     &exper->h_min, &exper->h_max, &exper->k_start, 
                     &exper->k_min, &exper->k_max, &exper->l_start, 
                     &exper->l_min, &exper->l_max ); 

             rec_flg |= GOT_REC_24;
             break;
          case 25:
             sscanf( read_in,
             "%d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d %d",
             &ijunk, &exper->n_absences,
             &exper->abs_matrix[0][0], &exper->abs_matrix[0][1], &exper->abs_matrix[0][2],
             &exper->abs_matrix[1][0], &exper->abs_matrix[1][1], &exper->abs_matrix[1][2],
             &exper->abs_matrix[2][0], &exper->abs_matrix[2][1], &exper->abs_matrix[2][2],
             &exper->abs_matrix[3][0], &exper->abs_matrix[3][1], &exper->abs_matrix[3][2],
             &exper->abs_matrix[4][0], &exper->abs_matrix[4][1], &exper->abs_matrix[4][2],
             &exper->abs_matrix[5][0], &exper->abs_matrix[5][1], &exper->abs_matrix[5][2] );

             rec_flg |= GOT_REC_25;
             break;
          case 26:
             sscanf( read_in, "%d %d %f %f %d %f %f %f %u", &ijunk,
                     &exper->intcr_rep_time, &exper->int_chk_sigma,
                     &exper->decay_fraction, &exper->orient_rep_count,
                     &exper->dev_ang, &exper->instr_const_1,
                     &exper->instr_const_2, &exper->xray_exp_time );

             rec_flg |= GOT_REC_26;
             break;
          case 27:
             sscanf( read_in,"%d %f %f %f %f %f %f %f %c %d %c", &ijunk,
                    &exper->bgsel, &exper->bgtime, &exper->bgdist,
                    &fjunk_27[0], &fjunk_27[1], &fjunk_27[2], &fjunk_27[3], 
                    &mystery1, &ijunk_27, &mystery2 );

             rec_flg |= GOT_REC_27;
                    
             break;
          case 31:
             sscanf( read_in, "%d %f %f %f %f %f %f", &ijunk, 
             &exper->orient_matrix[0][0], &exper->orient_matrix[0][1], &exper->orient_matrix[0][2],
             &exper->orient_matrix[1][0], &exper->orient_matrix[1][1], &exper->orient_matrix[1][2] );

             rec_flg |= GOT_REC_31;
             break;
          case 32:
             sscanf( read_in, "%d %f %f %f %f %7f%9f", &ijunk, 
             &exper->orient_matrix[2][0], &exper->orient_matrix[2][1], &exper->orient_matrix[2][2],
             &exper->lambda1, &exper->lambda2, &exper->attenuator );  

             rec_flg |= GOT_REC_32;

             break;
          default:  /* do nothing for the other records */
             break;
        }
    }

    rewind( cad4_file );
    return;  
}
