/* header file for processing a CAD4 data file.
 */
#define N_LIST         25         /* number of reflection in fcryst list */
#define CAD4_LINE_LEN  64
#define END_DATA_FILE  "$$$$"

#ifdef USING_PROFILES  /* use when you want to read in profiles for
                        * intensity data.
                        */         
#define N_DUMPS        96         /* an Enraf-Nonius magic number */
#endif

#define H    0
#define K    1
#define L    2

struct cad4_experimental {
      int           npi_pre,
                    scan_mode,
                    max_time,
                    dumpfl,
                    n_balf, 
                    n_fridl,
                    n_eqfl,
                    nr_psi,
                    h_start,
                    h_min,
                    h_max,
                    k_start,
                    k_min,
                    k_max,
                    l_start,
                    l_min,
                    l_max,
                    n_absences,
                    abs_matrix[6][3],
                    intcr_rep_time,
                    orient_rep_count,
                    i_unknown;
     unsigned int   xray_exp_time;
     float          instr_radius,
                    instr_const_1,
                    instr_const_2,
                    vertical_slit,
                    apeture_min,
                    apeture_max,
                    theta_min,
                    theta_max,
                    omega_term_a,
                    omega_term_b,
                    apeture_set_a,
                    apeture_set_b,
                    sigma_prescan,
                    sigma_desired,
                    psi_start,
                    psi_delta,
                    vect[3];
     float          int_chk_sigma,
                    decay_fraction,
                    dev_ang;
     float          lambda1,
                    lambda2,
                    attenuator,
                    orient_matrix[3][3];
     float          bgsel,
                    bgtime,
                    bgdist,
                    f_unknown[4];
     char           text[40],
                    psi_word[8],
                    mode_word[8],
                    seq[4];
    };

struct cad4_reflection {
       int           nrefl, 
                     fridl;
       unsigned int  xrayt;
       float         hkl[3],
                     npi,
                     bkg_left,
                     bkg_right,
                     peak,
                     psi,
                     theta,
                     phi_k,
                     omega_k,
                     kappa,
                     width;
      char           code[7];
#ifdef USING_PROFILES
      float          *profile;   /* in case profiles are wanted */
#endif
      };

/* prototypes */
void GetCAD4Experimental( FILE *cad4_file,
                          struct cad4_experimental *exper );
struct cad4_reflection   *GetCAD4Reflection( FILE *cad4_file,
                                             int *n_ref,
                                             int *i_dumps );
/* If profiles not wanted, set *i_dumps = NULL */
